"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.CheckMultipleDeclarationVisitor = void 0;
const visitor_1 = require("./visitor");
const ast_1 = require("../ast");
class CheckMultipleDeclarationVisitor extends visitor_1.Visitor {
    constructor() {
        super(...arguments);
        this.nameToType = new Map();
    }
    visit(node) {
        if (node instanceof ast_1.TypeDefinition) {
            const previousType = this.nameToType.get(node.name);
            if (previousType && JSON.stringify(previousType) !== JSON.stringify(node.type)) {
                throw new visitor_1.SemanticError(`Type '${node.name}' at ${node.location} is defined multiple times (also at ${previousType.location})`);
            }
            this.nameToType.set(node.name, node.type);
        }
        super.visit(node);
    }
}
exports.CheckMultipleDeclarationVisitor = CheckMultipleDeclarationVisitor;
