"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.analyse = void 0;
const _01_check_multiple_declaration_1 = require("./01_check_multiple_declaration");
const _02_give_struct_and_enum_names_1 = require("./02_give_struct_and_enum_names");
const _03_match_type_definitions_1 = require("./03_match_type_definitions");
const _04_check_dont_return_secret_1 = require("./04_check_dont_return_secret");
const _05_expand_spreads_1 = require("./05_expand_spreads");
const _06_check_empty_struct_or_enum_1 = require("./06_check_empty_struct_or_enum");
const _07_validate_recursive_types_1 = require("./07_validate_recursive_types");
const _08_collect_struct_and_enum_types_1 = require("./08_collect_struct_and_enum_types");
const _09_validate_annotations_1 = require("./09_validate_annotations");
const _10_check_duplicate_members_on_enum_1 = require("./10_check_duplicate_members_on_enum");
const ast_1 = require("../ast");
function analyse(root) {
    if (!root.errors.some(error => error.name === "Fatal")) {
        root.errors.push(new ast_1.ErrorNode("Fatal", new ast_1.VoidPrimitiveType()));
    }
    new _01_check_multiple_declaration_1.CheckMultipleDeclarationVisitor(root).process();
    new _02_give_struct_and_enum_names_1.GiveStructAndEnumNamesTransformer(root).process();
    new _03_match_type_definitions_1.MatchTypeDefinitionsVisitor(root).process();
    new _04_check_dont_return_secret_1.CheckDontReturnSecretVisitor(root).process();
    new _05_expand_spreads_1.ExpandSpreadsVisitor(root).process();
    new _06_check_empty_struct_or_enum_1.CheckEmptyStructOrEnumVisitor(root).process();
    new _07_validate_recursive_types_1.ValidateRecursiveTypes(root).process();
    new _08_collect_struct_and_enum_types_1.CollectStructAndEnumTypesVisitor(root).process();
    new _09_validate_annotations_1.ValidateAnnotationsVisitor(root).process();
    new _10_check_duplicate_members_on_enum_1.CheckDuplicatedMembersOnEnumVisitor(root).process();
}
exports.analyse = analyse;
